unit uScriptSite;

{
uScriptSite is a unit that impliments the IActiveScriptSite and
IActiveScriptSiteWindow interfaces as TActiveScriptSite and
TActiveScriptSiteWindow.  Use TActiveScriptSiteWindow to allow the
script to gain access to the user interface.  (Whatever that means...)

The TScriptSite and TScriptSiteWindow objects here were derived from
similar objects programmed by Ryan VanIderstine, David Zajac, and
Brian Dupras.  The basic design of the object follows Ryan's implimentation
closer than the other two - most notably the use of the string list as an
internal structure for holding the list of globals added to the script.
}

interface

uses  Windows, ComObj, ActiveX, ComServ, MsScript, AXScript, AXScriptHelp, Classes;

const
  Class_ScriptSite:TGUID       = '{9CD40281-CECA-11D0-B7E7-0000C03EEB7B}';
  Class_ScriptSiteWindow:TGUID = '{9CD40280-CECA-11D0-B7E7-0000C03EEB7B}';

type
  TScriptSite=class(TComObject, IActiveScriptSite)
  protected
    GlobalList:TStringList;
    FScriptDepth:Integer;
    FResult:OleVariant;
    procedure DeleteGlobalByIndex(const index:integer);
  public
    constructor Create;
    destructor  Destroy; override;
  public
    procedure AddGlobal(const ObjName:String; Obj:IUnknown);
    procedure DeleteGlobal(const ObjName:String);
    procedure ClearGlobals;
  public
    property  ScriptDepth:Integer read FScriptDepth write FScriptDepth;
    property  Result:OleVariant read FResult write FResult;
  protected

    //IActiveScriptSite methods
    function GetLCID(out plcid:TLCID):HResult; virtual; stdcall;
    function GetItemInfo(const pstrName:POleStr; dwReturnMask:DWord; out ppiunkItem:IUnknown; out ppti:ITypeInfo): HResult; virtual; stdcall;
    function GetDocVersionString(out pbstrVersion:TBStr):HResult; virtual; stdcall;
    function OnScriptTerminate(const pvarResult:OleVariant; const pexcepinfo:TExcepInfo):HResult; virtual; stdcall;
    function OnStateChange(ssScriptState:TScriptState):HResult; virtual; stdcall;
    function OnScriptError(const pscripterror:IActiveScriptError):HResult; virtual; stdcall;
    function OnEnterScript:HResult; virtual; stdcall;
    function OnLeaveScript:HResult; virtual; stdcall;
  end; //TScriptSite

type
  TScriptSiteWindow=class(TScriptSite, IActiveScriptSiteWindow)
  protected
    FWindowHandle:HWnd;
    FModeless:Boolean;
  public
    function GetWindow(out phWnd:HWND):HResult; virtual; stdcall;
    function EnableModeless(fEnable:BOOL):HResult; virtual; stdcall;
  public
    //IActiveScriptSiteWindow methods
    property  WindowHandle:HWnd read FWindowHandle write FWindowHandle;
    property  Modeless:Boolean read FModeless;
  end; //TScriptSiteWindow

implementation

uses  SysUtils, Dialogs;

constructor TScriptSite.Create;
begin
  inherited Create;
  GlobalList:=TStringList.Create;
  GlobalList.Sorted:=true;
end;

destructor  TScriptSite.Destroy;
begin
  ClearGlobals;
  try GlobalList.Free; except end;
  inherited Destroy;
end;

procedure TScriptSite.AddGlobal(const ObjName:String; Obj:IUnknown);
begin
  Obj._AddRef;
  GlobalList.AddObject(ObjName, pointer(Obj));
end;

procedure TScriptSite.DeleteGlobalByIndex(const index:integer);
var obj:IUnknown;
begin
  if (index>=0) then begin
    Obj:=IUnknown(pointer(GlobalList.Objects[index]));
    Obj._Release;
    GlobalList.Delete(index);
  end; //if global was found
end;

procedure TScriptSite.DeleteGlobal(const ObjName:String);
begin
  DeleteGlobalByIndex(GlobalList.IndexOf(ObjName));
end;

procedure TScriptSite.ClearGlobals;
begin
  if not Assigned(GlobalList) then exit;
  while (GlobalList.Count>0) do
    DeleteGlobalByIndex(0);
  GlobalList.Clear;
end;

function TScriptSite.GetLCID(out plcid:TLCID):HResult;
begin
  //The following C Macros return $409 which is US English, with SORT_DEFAULT.
  plcid := MakeLCID(MakeLangID(LANG_ENGLISH, SUBLANG_ENGLISH_US), SORT_DEFAULT);
  Result := S_OK;
end;

function TScriptSite.GetItemInfo(const pstrName: POleStr; dwReturnMask: DWord; out ppiunkItem: IUnknown; out ppti: ITypeInfo):HResult;
var
  s:String;
  i:Integer;
  Obj:IUnknown;
  ci:IProvideClassInfo;

begin
  try
    if ((dwReturnMask and ScriptInfo_IUnknown) <> 0)
      then pointer(ppiunkItem):=nil;   // use pointer to avoid auto _Release

    if ((dwReturnMask and ScriptInfo_ITypeInfo) <> 0)
      then pointer(ppti):=nil; // use pointer to avoid auto _Release

    s:=Uppercase(pstrName);
    i:=GlobalList.IndexOf(s);
    if (i<0) then begin
      Result := TYPE_E_ELEMENTNOTFOUND;
      Exit;
    end; //if global not found

    Result := S_OK;
    pointer(Obj):=GlobalList.Objects[i]; // as TAutoObject;
    try
      if ((dwReturnMask and ScriptInfo_IUnknown) <> 0) then begin
          Result:=Obj.QueryInterface(IUnknown, ppiunkItem);
          ppiunkItem._AddRef;
      end;

      if ((dwReturnMask and ScriptInfo_ITypeInfo) <> 0) then begin
        if (Obj.QueryInterface(IProvideClassInfo, ci) = 0) then begin
            OleCheck(ci.GetClassInfo(ppti));
            ppti._AddRef;
        end; //if QueryInterface(IProvideClassInfo)
      end; //if ITypeInfo
    except
      // swallow any exceptions
      Result := TYPE_E_ELEMENTNOTFOUND;
    end; //try/except

  except // in case it was called incorrectly
    Result := E_UNEXPECTED;
  end; //try/except
end;

function TScriptSite.GetDocVersionString(out pbstrVersion:TBStr):HResult;
begin
  Result:=E_NOTIMPL;
end;

function TScriptSite.OnScriptTerminate( const pvarResult: OleVariant; const pexcepinfo: TExcepInfo ): HResult;
begin
  FResult:=pvarResult;
  Result:=S_OK;
end;

function TScriptSite.OnStateChange( ssScriptState: TScriptState ): HResult;
begin
  Result:=S_OK;
end;


function TScriptSite.OnScriptError(const pscripterror:IActiveScriptError):HResult;
var dwCookie: DWORD;
    nLine:    ULONG;
    nChar:    LONGINT;
    ExcepInfo: TExcepInfo;
    SourceLine: LPWSTR;
    ErrorText:String;

begin
  ErrorText:='Script Error!';

  pscripterror.GetSourcePosition(dwCookie, nLine, nChar);
  ErrorText:=ErrorText+(format(#13'Context %d, Error in Line %d, Column %d',[dwCookie, nLine + 1, nChar + 1]));

  pscripterror.GetSourceLineText(SourceLine);
  if assigned(SourceLine) then begin
    ErrorText:=ErrorText+#13'Text: '+WideCharToString(SourceLine);
    SysFreeString(SourceLine);
  end;

  pscripterror.GetExceptionInfo(ExcepInfo);
  if ExcepInfo.bstrSource<>''
    then ErrorText:=ErrorText+#13'Source: ' + ExcepInfo.bstrSource;

  if ExcepInfo.bstrDescription<>''
    then ErrorText:=ErrorText+#13'Description: ' + ExcepInfo.bstrDescription;

  if ExcepInfo.bstrHelpFile<>''
    then ErrorText:=ErrorText+#13'HelpFile: ' + ExcepInfo.bstrHelpFile;

  ShowMessage(ErrorText);
  Result := S_OK;
end; //OnScriptError


function TScriptSite.OnEnterScript:HResult;
begin
  Inc(FScriptDepth);
  Result := S_OK;
end;

function TScriptSite.OnLeaveScript:HResult;
begin
  Dec(FScriptDepth);
  Result:=S_OK;
end;

function TScriptSiteWindow.GetWindow(out phWnd:HWND):HResult;
begin
  phWnd:=FWindowHandle;
  Result:=S_OK;
end;

function TScriptSiteWindow.EnableModeless(fEnable:BOOL):HResult;
begin
  FModeless:=fEnable;
  Result:=S_OK;
end;

initialization
  TComObjectFactory.Create(ComServer, TScriptSiteWindow, Class_ScriptSiteWindow, '', 'ActiveScript Site/Window Interface to Script Engine', ciMultiInstance );
end.

